import java.awt.*;
import javax.swing.*;
import java.awt.Color;
import java.util.Random;

public class DrawingPlane extends JPanel
{
    protected EntityPlane Plane; //класс-сущность
    public IAdditionalDrawingObject _airplaneWindow; //для дополнительной отрисовки

    public void SetAddEnum()
    {
        Random rnd = new Random();
        int numbEnum = rnd.nextInt(1, 4);
        _airplaneWindow.SetAddEnum(numbEnum);
    }

    public void SetFormEnum()
    {
        Random rnd = new Random();

        int numbEnum = rnd.nextInt(1, 4);

        if(numbEnum == 1)
        {
            _airplaneWindow = new DrawingAirplaneWindow();
        }

        if(numbEnum == 2)
        {
            _airplaneWindow = new DrawingTriangleAirplaneWindow();
        }

        if(numbEnum == 3)
        {
            _airplaneWindow = new DrawingRectAirplaneWindow();
        }
    }

    public EntityPlane GetPlane()
    {
        return Plane;
    }

    protected float _startPosX; //левая координата отрисовки
    protected float _startPosY; //верхняя координата отрисовки
    private Integer _pictureWidth = null; //ширина окна отрисовки
    private Integer _pictureHeight = null; //высота окна отрисовки
    protected int _airbusWidth = 50; //ширина отрисовки самолёта
    protected int _airbusHeight = 16; //высота отрисовки самолёта

    //конструктор
    public DrawingPlane(int speed, float weight, Color corpusColor)
    {
        Plane = new EntityPlane(speed, weight, corpusColor);
        SetFormEnum();
        SetAddEnum();
    }

    //конструктор
    public DrawingPlane(int speed, float weight, Color corpusColor, int planeWidth, int planeHeight)
    {
        this(speed, weight, corpusColor);
        _airbusWidth = planeWidth;
        _airbusHeight = planeHeight;
    }

    //установка координат позиции самолёта
    public void SetPosition(int x, int y, int width, int height)
    {
        if (x < 0 || y < 0 || width <= x + _airbusWidth || height <= y + _airbusHeight)
        {
            _pictureHeight = null;
            _pictureWidth = null;

            return;
        }

        _startPosX = x;
        _startPosY = y;
        _pictureWidth = width;
        _pictureHeight = height;
    }

    //изменение направления движения
    public void MoveTransport(Direction direction)
    {
        if (GetPlane() == null)
        {
            return;
        }

        switch (direction)
        {
            case Right: //вправо
                if (_startPosX + _airbusWidth + Plane.GetStep() < _pictureWidth)
                {
                    _startPosX += Plane.GetStep();
                }
                break;

            case Left: //влево
                if (_startPosX - _airbusWidth - Plane.GetStep() > 0)
                {
                    _startPosX -= Plane.GetStep();
                }
                break;

            case Up: //вверх
                if (_startPosY - _airbusHeight - Plane.GetStep() > 0)
                {
                    _startPosY -= Plane.GetStep();
                }
                break;

            case Down: //вниз
                if (_startPosY + _airbusHeight + Plane.GetStep() < _pictureHeight)
                {
                    _startPosY += Plane.GetStep();
                }
                break;
        }
    }

    public void DrawTransport(Graphics g)
    {
        if (_startPosX < 0 || _startPosY < 0 || _pictureHeight == null || _pictureWidth == null)
        {
            return;
        }

        if(GetPlane() == null){
            return;
        }

        Graphics2D g2d = (Graphics2D)g;

        //корпус
        g2d.setColor(Plane.GetColor());
        g2d.fillRect((int)_startPosX, (int)_startPosY + 10, 40, 10);

        //заливает фигуру
        g2d.setPaint(Color.BLACK);

        //крыло
        g2d.fillRect((int)_startPosX + 10, (int)_startPosY + 15, 20, 2);
        g2d.fillRect((int)_startPosX + 12, (int)_startPosY + 17, 16, 2);

        //заднее поперечное крыло
        g2d.fillOval((int)_startPosX - 3, (int)_startPosY + 7, 10, 5);

        //задние шасси
        g2d.fillRect((int)_startPosX + 10, (int)_startPosY + 21, 2, 2);
        g2d.fillRect((int)_startPosX + 12, (int)_startPosY + 23, 4, 4);
        g2d.fillRect((int)_startPosX + 8, (int)_startPosY + 23, 2, 4);

        //переднее шасси
        g2d.fillRect((int)_startPosX + 35, (int)_startPosY + 21, 2, 2);
        g2d.fillRect((int)_startPosX + 35, (int)_startPosY + 23, 4, 4);

        //хвостовое крыло
        g2d.drawLine((int)_startPosX, (int)_startPosY + 10, (int)_startPosX, (int)_startPosY);
        g2d.drawLine((int)_startPosX, (int)_startPosY, (int)_startPosX + 10, (int)_startPosY + 10);

        //нос самолёта
        g2d.drawLine((int)_startPosX + 40, (int)_startPosY + 15, (int)_startPosX + 50, (int)_startPosY + 16);
        g2d.drawLine((int)_startPosX + 40, (int)_startPosY + 10, (int)_startPosX + 47, (int)_startPosY + 15);
        g2d.drawLine((int)_startPosX + 40, (int)_startPosY + 20, (int)_startPosX + 50, (int)_startPosY + 15);

        //отрисовка иллюминаторов
        _airplaneWindow.DrawAirplaneWindow(Plane.GetColor(), g, _startPosX, _startPosY);
    }

    //смена границ формы отрисовки
    public void ChangeBorders(int width, int height)
    {
        _pictureWidth = width;
        _pictureHeight = height;

        if (_pictureWidth <= _airbusWidth || _pictureHeight <= _airbusHeight)
        {
            _pictureWidth = null;
            _pictureHeight = null;
            return;
        }

        if (_startPosX + _airbusWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _airbusWidth;
        }

        if (_startPosY + _airbusHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _airbusHeight;
        }
    }

    public float[] GetCurrentPosition()
    {
        return new float[]{_startPosX, _startPosY, _startPosX + _airbusWidth, _startPosY + _airbusHeight};
    }
}
